#import "MyImageController.h"

@implementation MyImageController

- (IBAction)setImage:(id)sender
{
	unsigned char* rawData;
	int i;

	//First store a Copy of the Image dragged onto the App.
	originalImage = [[ sender image ] copy ];
	
	//We're going to modify the current Image
	NSImage* currentImage;
	currentImage = [ sender image ] ;

	//We'r going to to this with with OpenCV. To create an OpenCV Image we need a "Size".
	//We'll get this from the image dragged onto the App.
	CVImageSize.width = [ currentImage size ].width;
	CVImageSize.height = [ currentImage size ].height;

	//create a picture with similar size, unsigned 8 bit dataformat, 1 colorchannel (greyscale)
	openCVImage = cvCreateImage (CVImageSize, IPL_DEPTH_8U, 1);

	NSLog(@"Image set to %d, %d.\n", CVImageSize.width, CVImageSize.height );

	//We' store the number of Bytes, as it will be usefull while copying between CV und Cocoa...
	length = CVImageSize.width* CVImageSize.height;

	//Let's get our fingers dirty and get the raw bytes.
	rawData = [[ currentImage bestRepresentationForDevice : nil ] bitmapData ];
	
	if (!rawData){
		NSLog(@"Woops, got no data!");
		return;
	}
	
	//Create some Greyscale Data.
	for (i = 0; i < length; i++){ //greyscale
		openCVImage->imageData[ i ] = (rawData[ 3*i + 0 ]+rawData[ 3*i + 1 ]+rawData[ 3*i + 2 ])/3;
	}
	
	//As a test that everything worked modify the contents of that NSImage...
	for (i = 0; i < length; i++){
		rawData [ i * 3 +0] = openCVImage->imageData[ i ];
		rawData [ i * 3 +1] = openCVImage->imageData[ i ];
		rawData [ i * 3 +2] = openCVImage->imageData[ i ];
	} 
	ready = true;
}

- (IBAction)setToOrig:(id)sender
{
	if (!ready)
		return;
	
	//We'll tell the NSImageView to display our original Image...
	[ myView setImage: originalImage ];
	NSLog(@"Orig set.\n");
}

- (IBAction)setToSobel:(id)sender
{
	if (!ready)
		return;
	
	int i;
	int SobelKernel;
	IplImage * work;
	IplImage * work2;

	//We'll demonstrate the Library by creating a Sobel-Filtered NS-Image.
	//Copy the original one, so we do not need to set Size and Stuff...
	NSImage * update = [ originalImage copy ];

	//Allocate some temporary OpenCV Images. Neccessary as Sobel has a signed 16 Bit
	//result but graphics hardware is only capable of displaying unsigned 8 Bit...
	work = cvCreateImage (CVImageSize, IPL_DEPTH_16S, 1);
	work2 = cvCreateImage (CVImageSize, IPL_DEPTH_8U, 1);

	//Get Filter-Kernel-Size from Drop-List
	SobelKernel = [ SobelKernelField intValue ];
	
	//Run the Filter. Result is written into work.
	cvSobel( openCVImage, work, 1, 1, SobelKernel);

	//Map to usigned 8 Bit, Write result to work2.
	cvConvertScale (work, work2, 1.0, 0.0);
	
	//get a Handle for the Data we're going to write.
	char* rawData = [[ update bestRepresentationForDevice : nil ] bitmapData ];
	
	if (!rawData){
		NSLog(@"Woops, got no data!");
		//Emergency cleanup!
		cvReleaseImage(&work);
		cvReleaseImage(&work2);
		return;
	}
	
	//Write our Result into the NSImage
	for (i = 0; i < length; i++){
		rawData [ i * 3 +0] = work2->imageData[ i ];
		rawData [ i * 3 +1] = work2->imageData[ i ];
		rawData [ i * 3 +2] = work2->imageData[ i ];
	}
	
	//Tell Viewer to display Fresh Image
	[ myView setImage: update ];
	
	//Clean up
	cvReleaseImage(&work);
	cvReleaseImage(&work2);
	
	NSLog(@"Zobel set.\n");
}

@end
